@@ -1,6 +1,6 @@
 /* Data definitions for internal representation of Bison's input.
 
-   Copyright (C) 1984, 1986, 1989, 1992, 2001-2007, 2009-2013 Free
+   Copyright (C) 1984, 1986, 1989, 1992, 2001-2007, 2009-2015 Free
    Software Foundation, Inc.
 
    This file is part of Bison, the GNU Compiler Compiler.
@@ -35,7 +35,7 @@
 
    The rules receive rule numbers 1 to NRULES in the order they are
    written.  More precisely Bison augments the grammar with the
-   initial rule, `$accept: START-SYMBOL $end', which is numbered 1,
+   initial rule, '$accept: START-SYMBOL $end', which is numbered 1,
    all the user rules are 2, 3 etc.  Each time a rule number is
    presented to the user, we subtract 1, so *displayed* rule numbers
    are 0, 1, 2...
@@ -61,7 +61,7 @@
    RULES[R].prec -- the symbol providing the precedence level of R.
 
    RULES[R].precsym -- the symbol attached (via %prec) to give its
-   precedence to R.  Of course, if set, it is equal to `prec', but we
+   precedence to R.  Of course, if set, it is equal to 'prec', but we
    need to distinguish one from the other when reducing: a symbol used
    in a %prec is not useless.
 
@@ -105,15 +105,15 @@
 # include "location.h"
 # include "symtab.h"
 
-# define ISTOKEN(i)	((i) < ntokens)
-# define ISVAR(i)	((i) >= ntokens)
+# define ISTOKEN(i)     ((i) < ntokens)
+# define ISVAR(i)       ((i) >= ntokens)
 
 extern int nsyms;
 extern int ntokens;
 extern int nvars;
 
 typedef int item_number;
-#define ITEM_NUMBER_MAX INT_MAX
+# define ITEM_NUMBER_MAX INT_MAX
 extern item_number *ritem;
 extern unsigned int nritems;
 
@@ -145,7 +145,7 @@ item_number_is_symbol_number (item_number i)
 
 /* Rule numbers.  */
 typedef int rule_number;
-#define RULE_NUMBER_MAX INT_MAX
+# define RULE_NUMBER_MAX INT_MAX
 extern rule_number nrules;
 
 static inline item_number
@@ -194,6 +194,7 @@ typedef struct
 
   location location;
   bool useful;
+  bool is_predicate;
 
   const char *action;
   location action_location;
@@ -202,34 +203,31 @@ typedef struct
 extern rule *rules;
 
 /* A function that selects a rule.  */
-typedef bool (*rule_filter) (rule *);
+typedef bool (*rule_filter) (rule const *);
 
-/* Return true IFF the rule has a `number' smaller than NRULES.  That is, it is
+/* Return true IFF the rule has a 'number' smaller than NRULES.  That is, it is
    useful in the grammar.  */
-bool rule_useful_in_grammar_p (rule *r);
+bool rule_useful_in_grammar_p (rule const *r);
 
-/* Return true IFF the rule has a `number' higher than NRULES.  That is, it is
+/* Return true IFF the rule has a 'number' higher than NRULES.  That is, it is
    useless in the grammar.  */
-bool rule_useless_in_grammar_p (rule *r);
+bool rule_useless_in_grammar_p (rule const *r);
 
 /* Return true IFF the rule is not flagged as useful but is useful in the
    grammar.  In other words, it was discarded because of conflicts.  */
-bool rule_useless_in_parser_p (rule *r);
+bool rule_useless_in_parser_p (rule const *r);
 
 /* Print this rule's number and lhs on OUT.  If a PREVIOUS_LHS was
    already displayed (by a previous call for another rule), avoid
    useless repetitions.  */
-void rule_lhs_print (rule *r, symbol *previous_lhs, FILE *out);
-void rule_lhs_print_xml (rule *r, FILE *out, int level);
+void rule_lhs_print (rule const *r, symbol const *previous_lhs, FILE *out);
+void rule_lhs_print_xml (rule const *r, FILE *out, int level);
 
 /* Return the length of the RHS.  */
-int rule_rhs_length (rule *r);
+size_t rule_rhs_length (rule const *r);
 
 /* Print this rule's RHS on OUT.  */
-void rule_rhs_print (rule *r, FILE *out);
-
-/* Print this rule on OUT.  */
-void rule_print (rule *r, FILE *out);
+void rule_rhs_print (rule const *r, FILE *out);
 
 
 
@@ -253,7 +251,7 @@ size_t ritem_longest_rhs (void);
 
 /* Print the grammar's rules that match FILTER on OUT under TITLE.  */
 void grammar_rules_partial_print (FILE *out, const char *title,
-				  rule_filter filter);
+                                  rule_filter filter);
 
 /* Print the grammar's useful rules on OUT.  */
 void grammar_rules_print (FILE *out);
@@ -264,8 +262,8 @@ void grammar_rules_print_xml (FILE *out, int level);
 void grammar_dump (FILE *out, const char *title);
 
 /* Report on STDERR the rules that are not flagged USEFUL, using the
-   MESSAGE (which can be `rule useless in grammar' when invoked after grammar
-   reduction, or `rule useless in parser due to conflicts' after conflicts
+   MESSAGE (which can be 'rule useless in grammar' when invoked after grammar
+   reduction, or 'rule useless in parser due to conflicts' after conflicts
    were taken into account).  */
 void grammar_rules_useless_report (const char *message);
 
